/*
 * $Id: OrderDAO.java,v 1.11 2000/10/06 15:53:33 lblair Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.order.ejb;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.sql.DataSource;
import com.sun.j2ee.blueprints.customer.util.JNDINames;

import com.sun.j2ee.blueprints.customer.util.Debug;
import com.sun.j2ee.blueprints.customer.util.DatabaseNames;
import com.sun.j2ee.blueprints.customer.util.Address;
import com.sun.j2ee.blueprints.customer.util.Calendar;
import com.sun.j2ee.blueprints.customer.util.CreditCard;
import com.sun.j2ee.blueprints.customer.order.model.LineItem;
import com.sun.j2ee.blueprints.customer.order.ejb.OrderDAOException;

/**
 * This class encapsulates all the SQL calls made by
 * the Order EJB.This class provides two abstract methods,
 * namely abstract void create( Connection con) and
 * abstract void store(Connection con).The implementation of
 * these methods is provided by the subclasses representing
 * a particular database vendor.This was done primarily
 * to accomodate the differences between the SQL supported by
 * different vendors. So for each different DB there would be
 * a OrderDAOXXX.java class that encapsulates the DB specific
 * code.
 */
public abstract class OrderDAO {

    protected int orderId;
    protected Collection lineItems;
    protected Address shipToAddr;
    protected Address billToAddr ;
    protected String shipToFirstName;
    protected String shipToLastName;
    protected String billToFirstName;
    protected String billToLastName;
    protected CreditCard chargeCard;
    protected String carrier;
    protected String userId;
    protected Calendar orderDate;
    protected String status;
    protected double totalPrice;


    protected Connection dbConnection = null;
    private   DataSource datasource   = null;

    public OrderDAO () throws OrderDAOException {
        this.orderId = 0;
        this.orderDate = Calendar.getInstance();
        this.totalPrice = 0;

        try {
            InitialContext ic = new InitialContext();
            datasource  = (DataSource) ic.lookup(JNDINames.ESTORE_DATASOURCE);
        } catch (NamingException ne) {
            throw new OrderDAOException("NamingException while looking" +
                                               " up DataSource Connection " +
                                             JNDINames.ESTORE_DATASOURCE +
                                                    ": \n" + ne.getMessage());
        }
    }


    public abstract void create() throws OrderDAOException;

    public abstract void store() throws OrderDAOException;

    // get and set methods for the instance variables

    public int getOrderId() {
        return orderId;
    }
    public Collection getLineItems() {
        return lineItems;
    }
    public Address getShipToAddr() {
        return shipToAddr;
    }

    public Address getBillToAddr() {
        return billToAddr;
    }
    public String getBillToFirstName() {
        return billToFirstName;
    }
    public String getBillToLastName() {
        return billToLastName;
    }
    public String getShipToFirstName() {
        return shipToFirstName;
    }
    public String getShipToLastName() {
        return shipToLastName;
    }
    public CreditCard getChargeCard() {
        return chargeCard;
    }
    public String getCarrier() {
        return carrier;
    }
    public String getUserId() {
        return userId;
    }
    public Calendar getOrderDate() {
        return orderDate;
    }
    public String getStatus() {
        return status;
    }
    public double getTotalPrice() {
        return totalPrice;
    }

    public void setOrderId( int orderId) {
        this.orderId = orderId;
    }
    public void setLineItems( Collection lineItems) {
        this.lineItems = lineItems;
    }
    public void setShipToAddr(Address shipToAddr) {
        this.shipToAddr = shipToAddr;
    }
    public void setBillToAddr(Address billToAddr) {
        this.billToAddr =  billToAddr;
    }
    public void setShipToFirstName(String shipToFirstName) {
        this.shipToFirstName = shipToFirstName;
    }
    public void setShipToLastName(String shipToLastName) {
        this.shipToLastName = shipToLastName;
    }
    public void setBillToFirstName(String billToFirstName) {
        this.billToFirstName = billToFirstName;
    }
    public void setBillToLastName(String billToLastName) {
        this.billToLastName = billToLastName;
    }
    public void setChargeCard(CreditCard chargeCard) {
        this.chargeCard = chargeCard;
    }
    public void setCarrier(String carrier) {
        this.carrier = carrier;
    }
    public void setUserId(String userId) {
        this.userId = userId;
    }
    public void setOrderDate( Calendar orderDate) {
        this.orderDate = orderDate;
    }
    public void setStatus(String status) {
        this.status = status;
    }
    public void setTotalPrice( double totalPrice) {
        this.totalPrice = totalPrice;
    }

    public void load() throws OrderDAOException {
        selectOrder();
        selectLineItem();
        selectOrderStatus();
    }
    public void remove() throws OrderDAOException {
        deleteLineItem();
        deleteOrderStatus();
        deleteOrder();
    }
    public Integer findByPrimaryKey() throws OrderDAOException {
        if (orderIdExists(orderId))
            return (new Integer(orderId));
        throw new OrderDAOException("primary key not found:" + orderId);
    }
    public Collection findUserOrders() throws OrderDAOException {
        return(getOrderIds(userId));
    }


    private boolean orderIdExists (int orderId) throws OrderDAOException {
        String queryStr ="SELECT orderid FROM " +
                DatabaseNames.ORDER_TABLE + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);
            if ( !result.next() ) {
                return false;
            } else {
                return true;
            }
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while searching for " +
                        "order ID " + orderId + ": \n" + se);
        } finally {
            closeConnection();
        }
    }

    private Collection getOrderIds(String userId) throws OrderDAOException {
        String queryStr ="SELECT orderid FROM " +
            DatabaseNames.ORDER_TABLE
                + " WHERE userid = " + "'" + userId.trim() +"'";
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            ArrayList orderIdList = new ArrayList();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);
            if ( !result.next() ) {
                throw new OrderDAOException("No Orders found for user: " +
                                                                userId);
            } else {
                do {
                    orderIdList.add( new Integer(result.getInt(1)));
                } while (result.next());
                return orderIdList;
            }
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while searching for " +
                        "orders for user " + userId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void selectOrder() throws OrderDAOException {
        String queryStr = "SELECT "
            + "userid,orderdate,shipaddr1,shipaddr2,"
            + "shipcity,shipstate,shipzip,shipcountry,"
            + "billaddr1,billaddr2,billcity,billstate,billzip,billcountry,"
            + "courier,totalprice,"
            + "shiptofirstname,shiptolastname,"
            + "billtofirstname,billtolastname,"
            + "creditcard,exprdate,cardtype"
            + " FROM " + DatabaseNames.ORDER_TABLE
        + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);

            if ( !result.next() )
                throw new OrderDAOException("No record for primary key " +
                                   orderId);
            int i = 1;
            userId = result.getString(i++);
            orderDate.setTime(result.getDate(i++));

            String street1 = result.getString(i++);
            String street2 = result.getString(i++);
            String city = result.getString(i++);
            String state = result.getString(i++);
            String zip = result.getString(i++);
            String country = result.getString(i++);
            shipToAddr = new Address(street1,street2,city,state,zip,country);

            street1 = result.getString(i++);
            street2 = result.getString(i++);
            city = result.getString(i++);
            state = result.getString(i++);
            zip = result.getString(i++);
            country = result.getString(i++);
            billToAddr = new Address(street1,street2,city,state,zip,country);

            carrier = result.getString(i++);
            totalPrice = result.getFloat(i++);

            shipToFirstName = result.getString(i++);
            shipToLastName = result.getString(i++);
            billToFirstName = result.getString(i++);
            billToLastName = result.getString(i++);

            String cardNo = result.getString(i++);
            String expiryDate = result.getString(i++);
            String cardType = result.getString(i++);
            chargeCard = new CreditCard(cardNo,cardType,expiryDate);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while searching for " +
                        "order ID " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void selectLineItem() throws OrderDAOException {

        String queryStr = "SELECT "+
            "linenum,itemid,quantity,unitprice"
            + " FROM " + DatabaseNames.LINE_ITEM_TABLE
        + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ArrayList lineItemList = new ArrayList();
            ResultSet result = stmt.executeQuery(queryStr);
            if ( !result.next() )
                throw new SQLException("No Line Items for orderId: " +
                                   orderId);
            do {
                int lineNo = result.getInt(1);
                String itemNo = result.getString(2);
                int qty = result.getInt(3);
                double unitPrice = result.getFloat(4);
                LineItem LI = new LineItem(itemNo, qty, unitPrice, lineNo);
                // add the line items to the line item vector
                lineItemList.add(LI);
            } while (result.next());
            lineItems = lineItemList;
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while searching for " +
                        "orders by status : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void selectOrderStatus () throws OrderDAOException {

        String queryStr = "SELECT "
            + "status" + " FROM " + DatabaseNames.ORDER_STATUS_TABLE
                + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);
            if ( !result.next() )
                throw new OrderDAOException(
                        "Order status not found for orderId: " + orderId);
            status = result.getString(1);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while searching for " +
                        "status of order " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void deleteOrder() throws OrderDAOException {
        String queryStr = "DELETE FROM " + DatabaseNames.ORDER_TABLE
                + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);

            if ( resultCount != 1 )
                throw new OrderDAOException
                ("ERROR deleteing order from ORDER_TABLE!! resultCount = " +
                                         resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while deleting " +
                        "orders ID " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void deleteLineItem() throws OrderDAOException {
        String queryStr = "DELETE FROM " + DatabaseNames.LINE_ITEM_TABLE
                        + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);
            if ( resultCount < 1 )
                throw new OrderDAOException
                    ("ERROR deleteing line items!! resultCount = " +
                                         resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while removing LItem " +
                        "order ID " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    private void deleteOrderStatus() throws OrderDAOException {
        String queryStr = "DELETE FROM " + DatabaseNames.ORDER_STATUS_TABLE
                    + " WHERE orderid = " + orderId;
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);
            if ( resultCount < 1 )
                throw new SQLException
                    ("ERROR deleteing order status!! resultCount = " +
                         resultCount);
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while deleting " +
                        "orders by status; Id " + orderId + " : \n" + se);
        } finally {
            closeConnection();
        }
    }

    protected boolean isValidData () {
        if ( (userId == null) ||
             ( lineItems  == null) ||
             ( lineItems.size()  == 0) ||
             ( orderDate  == null) ||
             (shipToAddr.getStreetName1() == null) ||
             (shipToAddr.getCity() == null) ||
             (shipToAddr.getState() == null) ||
             (shipToAddr.getZipCode() == null) ||
             (shipToAddr.getZipCode() == null) ||
             (shipToAddr.getCountry() == null) ||
             (billToAddr.getStreetName1() == null) ||
             (billToAddr.getCity() == null) ||
             (billToAddr.getState() == null) ||
             (billToAddr.getZipCode() == null) ||
             (billToAddr.getCountry() == null) ||
             (shipToFirstName == null) ||
             (shipToLastName == null) ||
             (billToFirstName == null) ||
             (billToLastName == null) ||
             (carrier == null) ||
             (chargeCard.getCardNo() == null) ||
             (chargeCard.getCardType() == null) ||
             (chargeCard.getExpiryDateString() == null) )

            return false;
        else
            return true;
    }

    protected void getDBConnection() throws OrderDAOException {
        try {
            dbConnection = datasource.getConnection();
        } catch (SQLException se) {
            throw new OrderDAOException("SQLException while getting " +
                                      "DB connection : \n" + se.getMessage());
        }
    }

    protected void closeConnection() throws OrderDAOException {
        try {
            if (dbConnection!= null && !dbConnection.isClosed()) {
                dbConnection.close();
            }
        } catch(SQLException se) {
            throw new OrderDAOException("SQLException while closing " +
                                "DB connection : \n" + se.getMessage());
        }
    }
}

