/*
 * $Id: CatalogModel.java,v 1.14 2000/09/30 02:13:43 brydon Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.shoppingcart.catalog.model;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Locale;
import javax.ejb.FinderException;

import com.sun.j2ee.blueprints.shoppingcart.util.Debug;
import com.sun.j2ee.blueprints.shoppingcart.catalog.model.CatalogDAOException;
import com.sun.j2ee.blueprints.shoppingcart.catalog.model.CatalogDAO;
import com.sun.j2ee.blueprints.shoppingcart.catalog.model.ListChunk;

/**
 * This class implements the CatalogModel interface in order to
 * provide methods to browse and search the pet's catalog in the
 * Java Pet Store Demo. This class uses the CatalogDAO object to
 * access the data base for the pet store.
 * Users of this object must call the init method before invoking methods
 *
 * @see com.sun.j2ee.blueprints.shoppingcart.catalog.model.CatalogDAO
 */
public class CatalogModel {

    protected CatalogDAO dao;

    public CatalogModel() { }

    /**
     * Users of CatalogModel must call the init method
     * before invoking other methods
     */
    public void init()  throws RuntimeException {
        try {
            dao = new CatalogDAO();
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public void destroy() {
        dao = null;
    }

    public Category getCategory(String categoryId, Locale locale)
                                                     throws RuntimeException {
        try {
            return dao.getCategory(categoryId, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public ListChunk getCategories(int stIndex, int count, Locale locale)
                                                         throws RuntimeException {
        try {
            return dao.getCategories(stIndex, count, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public ListChunk getProducts(String categoryId,
                  int startIndex, int count, Locale locale) throws RuntimeException {
        try {
            return dao.getProducts(categoryId, startIndex, count, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public Product getProduct(String productId, Locale locale)
                                                      throws RuntimeException {
        try {
            return dao.getProduct(productId, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public ListChunk getItems(String productId, int startIndex,
                                int count, Locale locale) throws RuntimeException {
        try {
            return dao.getItems(productId, startIndex, count, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public Item getItem(String itemId, Locale locale) throws RuntimeException {
       try {
            return dao.getItem(itemId, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }

    public ListChunk searchProducts(Collection keyWords,
                   int startIndex, int count, Locale locale) throws RuntimeException {
        try {
            return dao.searchProducts(keyWords, startIndex, count, locale);
        } catch (CatalogDAOException se) {
            throw new RuntimeException(se.getMessage());
        }
    }
}
