/*
 * $Id: MailAction.java,v 1.11 2000/10/04 02:01:08 vijaysr Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.petstore.control.ejb;

import java.rmi.RemoteException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import com.sun.j2ee.blueprints.mail.ejb.EMailMessage;
import com.sun.j2ee.blueprints.mail.ejb.SendMailException;

import com.sun.j2ee.blueprints.customer.account.model.AccountModel;
import com.sun.j2ee.blueprints.customer.order.model.OrderModel;
import com.sun.j2ee.blueprints.customer.customer.ejb.Customer;
import com.sun.j2ee.blueprints.customer.customer.ejb.CustomerHome;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.util.EJBUtil;

import com.sun.j2ee.blueprints.mail.ejb.Mailer;

/**
 * This class encapsulates all the logic to build an EMail Message.
 * It uses the Order EJB and Account EJB to get the info to build
 * and send a message.
 *
 */
public class MailAction {

    public MailAction( ) {
    }

     /**
     * This method creates an email confirmation message
     * for an order and sends an email to the customer.
     * The Order EJB and Account EJB are accessed to
     * create the email message content.
     *
     * @param  order    is the orderEJB used to generate content
     *                  for the email message for a particular
     *                  customer order
     * @return          an EMailMessage object containing all info
     *                  to send the email.
     */
    public void sendConfirmationMessage(int orderId) {
        EMailMessage eMess = null;
        CustomerHome home = null;
        Customer cust = null;
        OrderModel orderDetails = null;
        AccountModel accountDetails = null;

        try {
                home = EJBUtil.getCustomerHome();
                cust = home.create();
        } catch (CreateException ce) {
                throw new EJBException (ce);
        } catch (RemoteException re) {
                throw new EJBException (re);
        } catch (javax.naming.NamingException ne) {
                throw new EJBException (ne);
        }

        try {
            orderDetails = cust.getOrderDetails(orderId);
            accountDetails = cust.getAccountDetails();
            if (accountDetails == null) {
                Debug.print("Can't find user corresponding to the order " +
                            orderDetails.getOrderId()+ "!");
            }

            //params:mail subjectLine, message contents, emailAddress
            eMess = new EMailMessage(createSubjectLine(orderDetails),
                           createMessage(orderDetails, accountDetails),
                           accountDetails.getContactInformation().getEMail());
            Mailer mailer = EJBUtil.createMailerEJB();
            mailer.sendMail(eMess);
        } catch (SendMailException me) {
            Debug.println("Send Mail Exception : " + me);
        } catch (RemoteException re) {
           Debug.println("Error trying to lookup Mail EJB");
        } catch (NamingException ne) {
           Debug.println("Error trying to lookup Mail EJB");
        } catch (CreateException ce) {
           Debug.println("Error trying to lookup Mail EJB");
        }
    }


    /**
     *  Builds the email message contents
     */
    private String createMessage(OrderModel orderDetails,
                                      AccountModel accountDetails) {
      StringBuffer msg = new StringBuffer();
      msg.append("This mail is a confirmation for your order# " +
                                orderDetails.getOrderId() + "\n");
      msg.append("Please save it for your records.\n");
      msg.append("Your order will be shipped to:\n");
      msg.append(orderDetails.getBillToFirstName() + " ");
      msg.append(orderDetails.getShipToLastName() + "\n");

      msg.append("The total cost of your order is $" +
                            orderDetails.getTotalPrice() + "\n");

      return msg.toString();
    }

    /**
     *  Builds the subject line for the  email message
     */
    private String createSubjectLine(OrderModel orderDetails) {
      StringBuffer msg = new StringBuffer();
      msg.append("Your order#" + orderDetails.getOrderId());

      return msg.toString();
    }

}
