/*
 * $Id: ProfileMgrEJB.java,v 1.18 2000/10/05 05:55:03 brydon Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.petstore.personalize.profilemgr.ejb;

import java.rmi.RemoteException;
import javax.naming.NamingException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;

import javax.ejb.EntityBean;
import javax.ejb.EntityContext;

import com.sun.j2ee.blueprints.petstore.personalize.util.ExplicitInformation;
import com.sun.j2ee.blueprints.petstore.personalize.util.EJBUtil;
import com.sun.j2ee.blueprints.petstore.personalize.profilemgr.model.ProfileMgrModel;
import com.sun.j2ee.blueprints.petstore.personalize.profilemgr.ejb.ProfileMgrDAOException;

import com.sun.j2ee.blueprints.util.tracer.Debug;

/**
 * Implementation of profile manager as an Entity Bean
 */
public class ProfileMgrEJB implements EntityBean {

    private String userId;
    private ExplicitInformation eInfo;

    private ProfileMgrDAO profileMgrDao;
    private EntityContext context;

    public ProfileMgrEJB() {}

    public String ejbCreate (String userId, ExplicitInformation eInfo)
                                throws CreateException {

        // set the instance data
        this.userId = userId;
        this.eInfo = eInfo;

        try {
            ProfileMgrDAO dao = getDAO();
            dao.setUserId(userId);
            dao.setExplicitInformation(eInfo);
            dao.create();
            return (userId);
        } catch (ProfileMgrDAOException se) {
            context.setRollbackOnly();
            throw new CreateException("ProfileMgrEJB: create failed : " +
                                            se.getMessage());
        }
    }

    public void ejbRemove() throws RemoveException {
        try {
            ProfileMgrDAO dao = getDAO();
            dao.setUserId(this.userId);
            dao.remove();
        } catch (ProfileMgrDAOException se) {
            context.setRollbackOnly();
            throw new RemoveException("ProfileMgrEJB: remove failed : " +
                                            se.getMessage());
        }
    }

    public void setEntityContext(EntityContext ec) {
        context = ec;
    }

    public void ejbLoad() {
        ProfileMgrDAO dao;
        try {
            dao = getDAO();
            dao.setUserId(this.userId);
            dao.load();
        } catch (ProfileMgrDAOException se) {
            throw new EJBException("ProfileMgrDAOException while load : " +
                                            se.getMessage());
        }
        userId = dao.getUserId();
        eInfo = dao.getExplicitInformation();
    }

    public void ejbStore() {
        try {
            ProfileMgrDAO dao = getDAO();
            dao.setUserId(this.userId);
            dao.setExplicitInformation(eInfo);
            dao.store();
        } catch (ProfileMgrDAOException se) {
            throw new EJBException("ProfileMgrDAOException while store : "+
                                                           se.getMessage());
        }
    }

    public String ejbFindByPrimaryKey(String key) throws FinderException {
        try {
            ProfileMgrDAO dao = getDAO();
            dao.setUserId(key);
            this.userId = dao.findByPrimaryKey();
            return this.userId;
        } catch (ProfileMgrDAOException se) {
            throw new FinderException(
                "ProfileMgrEJB: Exception in find by primary key : " +
                                                      se.getMessage());
        }
    }

    public void unsetEntityContext() {}

    public void ejbActivate() {}

    public void ejbPassivate() {}

    public void ejbPostCreate(String userId, ExplicitInformation eInfo)
        throws DuplicateKeyException,CreateException {
    }

    // business methods
    public void updateExplicitInformation(ExplicitInformation eInfo) {
        this.eInfo = eInfo;
    }

    public ProfileMgrModel getDetails() {
        return(new ProfileMgrModel(userId, eInfo));
    }

    private ProfileMgrDAO getDAO() throws ProfileMgrDAOException {
        if(profileMgrDao == null) {
            profileMgrDao = new ProfileMgrDAO();
        }
        return profileMgrDao;
    }
}

